@@ -1,5 +1,8 @@
 #!/usr/bin/env python
 """KvV2 methods module."""
+
+import warnings
+
 from hvac import exceptions, utils
 from hvac.api.vault_api_base import VaultApiBase
 
@@ -69,12 +72,44 @@ class KvV2(VaultApiBase):
         )
         return self._adapter.get(url=api_path)
 
-    def read_secret(self, path, mount_point=DEFAULT_MOUNT_POINT):
-        return self.read_secret_version(path, mount_point=mount_point)
-
-    def read_secret_version(self, path, version=None, mount_point=DEFAULT_MOUNT_POINT):
+    def read_secret(
+        self, path, mount_point=DEFAULT_MOUNT_POINT, raise_on_deleted_version=None
+    ):
         """Retrieve the secret at the specified location.
 
+        Equivalent to calling read_secret_version with version=None.
+
+        Supported methods:
+            GET: /{mount_point}/data/{path}. Produces: 200 application/json
+
+
+        :param path: Specifies the path of the secret to read. This is specified as part of the URL.
+        :type path: str | unicode
+        :param mount_point: The "path" the secret engine was mounted on.
+        :type mount_point: str | unicode
+        :param raise_on_deleted_version: Changes the behavior when the requested version is deleted.
+            If True an exception will be raised.
+            If False, some metadata about the deleted secret is returned.
+            If None (pre-v3), a default of True will be used and a warning will be issued.
+        :type raise_on_deleted_version: bool
+        :return: The JSON response of the request.
+        :rtype: dict
+        """
+        return self.read_secret_version(
+            path,
+            mount_point=mount_point,
+            raise_on_deleted_version=raise_on_deleted_version,
+        )
+
+    def read_secret_version(
+        self,
+        path,
+        version=None,
+        mount_point=DEFAULT_MOUNT_POINT,
+        raise_on_deleted_version=None,
+    ):
+        """Retrieve the secret at the specified location, with the specified version.
+
         Supported methods:
             GET: /{mount_point}/data/{path}. Produces: 200 application/json
 
@@ -85,19 +120,52 @@ class KvV2(VaultApiBase):
         :type version: int
         :param mount_point: The "path" the secret engine was mounted on.
         :type mount_point: str | unicode
+        :param raise_on_deleted_version: Changes the behavior when the requested version is deleted.
+            If True an exception will be raised.
+            If False, some metadata about the deleted secret is returned.
+            If None (pre-v3), a default of True will be used and a warning will be issued.
+        :type raise_on_deleted_version: bool
         :return: The JSON response of the request.
         :rtype: dict
         """
+
+        if raise_on_deleted_version is None:
+            msg = (
+                "The raise_on_deleted_version parameter will change its default value to False in hvac v3.0.0. "
+                "The current default of True will preserve previous behavior. "
+                "To use the old behavior with no warning, explicitly set this value to True. "
+                "See https://github.com/hvac/hvac/pull/907"
+            )
+            warnings.warn(
+                message=msg,
+                category=DeprecationWarning,
+                stacklevel=2,
+            )
+            raise_on_deleted_version = True
+
         params = {}
         if version is not None:
             params["version"] = version
         api_path = utils.format_url(
             "/v1/{mount_point}/data/{path}", mount_point=mount_point, path=path
         )
-        return self._adapter.get(
-            url=api_path,
-            params=params,
-        )
+        try:
+            return self._adapter.get(
+                url=api_path,
+                params=params,
+            )
+        except exceptions.InvalidPath as e:
+            if not raise_on_deleted_version:
+                try:
+                    if (
+                        e.json is not None
+                        and e.json["data"]["metadata"]["deletion_time"] != ""
+                    ):
+                        return e.json
+                except KeyError:
+                    pass
+
+            raise
 
     def create_or_update_secret(
         self, path, secret, cas=None, mount_point=DEFAULT_MOUNT_POINT
@@ -359,6 +427,7 @@ class KvV2(VaultApiBase):
         cas_required=None,
         delete_version_after="0s",
         mount_point=DEFAULT_MOUNT_POINT,
+        custom_metadata=None,
     ):
         """Updates the max_versions of cas_required setting on an existing path.
 
@@ -380,6 +449,8 @@ class KvV2(VaultApiBase):
         :type delete_version_after: str
         :param mount_point: The "path" the secret engine was mounted on.
         :type mount_point: str | unicode
+        :param custom_metadata: A dictionary of key/value metadata to describe the secret. Requires Vault 1.9.0 or greater.
+        :type custom_metadata: dict
         :return: The response of the request.
         :rtype: requests.Response
         """
@@ -397,6 +468,15 @@ class KvV2(VaultApiBase):
                 )
                 raise exceptions.ParamValidationError(error_msg)
             params["cas_required"] = cas_required
+        if custom_metadata is not None:
+            if not isinstance(custom_metadata, dict):
+                error_msg = (
+                    "dict expected for custom_metadata param, {type} received".format(
+                        type=type(custom_metadata)
+                    )
+                )
+                raise exceptions.ParamValidationError(error_msg)
+            params["custom_metadata"] = custom_metadata
         api_path = utils.format_url(
             "/v1/{mount_point}/metadata/{path}", mount_point=mount_point, path=path
         )
